/***************************************************************************
                          mastercd.cpp  -  description
                             -------------------
    begin                : Tue Mar 6 2001
    copyright            : (C) 2001 by Kai Heitkamp
    email                : koncd@kai-heitkamp.de
 ***************************************************************************/

#include <ctype.h>
#include <stdlib.h>

#include <qlineedit.h>
#include <qpushbutton.h>
#include <qcheckbox.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qprogressbar.h>
#include <qmultilineedit.h>

#include <kapp.h>
#include <kfiledialog.h>
#include <kconfig.h>
#include <kmessagebox.h>
#include <klocale.h>

#include "mastercd.h"
#include "imagetype.h"

MasterCD::MasterCD(QWidget *parent, const char *name ) : MasterCD_GUI(parent,name) {

	//Read config
	KConfig *config = kapp->config();
	config->setGroup( "Config - mastercd" );
	QCheckBox_BootableCD->setChecked( config->readBoolEntry("QCheckBox_BootableCD") );
	QCheckBox_Multisession->setChecked( config->readBoolEntry("QCheckBox_Multisession") );
	QCheckBox_LeaveImage->setChecked( config->readBoolEntry("QCheckBox_LeaveImage") );
	QCheckBox_ignoresize->setChecked( config->readBoolEntry("QCheckBox_ignoresize") );
	QCheckBox_Simulate->setChecked( config->readBoolEntry("QCheckBox_Simulate") );
	QCheckBox_nofix->setChecked( config->readBoolEntry("QCheckBox_nofix") );
	QCheckBox_EjectCD->setChecked( config->readBoolEntry("QCheckBox_EjectCD") );
	QCheckBox_Force->setChecked( config->readBoolEntry("QCheckBox_Force") );

	QComboBox_CDRWBlanking->setCurrentItem( config->readNumEntry("QComboBox_CDRWBlanking") );
	QComboBox_WriterSpeed->setCurrentItem( config->readNumEntry("QComboBox_WriterSpeed") );

	QLineEdit_VolumeID->setText( config->readEntry( "QLineEdit_VolumeID" ) );
	QLineEdit_Publisher->setText( config->readEntry( "QLineEdit_Publisher" ) );
	QLineEdit_Preparer->setText( config->readEntry( "QLineEdit_Preparer" ) );
	QLineEdit_AppID->setText( config->readEntry( "QLineEdit_AppID" ) );

	if ( config->readEntry( "QLineEdit_ImageFile" ) == "" ) {
		QLineEdit_ImageFile->setText( "/tmp/cd.iso" );
	}
	else {
	QLineEdit_ImageFile->setText( config->readEntry( "QLineEdit_ImageFile" ) );
	}

	QLineEdit_BootImage->setText( config->readEntry( "" ) );

	if ( strlen( QLineEdit_BootImage->text() ) ) {
		QPushButton_BootImage->setEnabled( true );
		QLineEdit_BootImage->setEnabled( true );
		QCheckBox_BootableCD->setChecked( true );
	}
	else {
		QCheckBox_BootableCD->setChecked( false );
	}

	//Read Image-Type from config file and display it
	config->setGroup("Image-Type");
	QLabel_ImageType->setText( config->readEntry( "Name" ) );

	//Check for RW-Device
	config->setGroup( "CD-Writer" );
	if ( config->readEntry( "isRW" ) == "false" )
	{
	QComboBox_CDRWBlanking->clear();	
	QComboBox_CDRWBlanking->insertItem(i18n( "None RW !" ));
	QComboBox_CDRWBlanking->setEnabled( false );
	}
}

MasterCD::~MasterCD(){
	existsSession = false;
	MS_Start = 0;
	MS_End = 0;
	FS_Size = 0;
}

//'SourceDir-Button' clicked
void MasterCD::slot_setSourceDir(){
	QString dir = KFileDialog::getExistingDirectory();

	if ( !dir ) return;

	int trackcount = 0;
	int tc = QListBox_SourceDirs->count();
	do {
			if ( dir == QListBox_SourceDirs->text( trackcount ) ){
				KMessageBox::error( 0, i18n("Master CD:\nThe dir exists already in the tracklist !") );
				return;
			}
		trackcount++;
	}
	while ( trackcount < tc );

//	if ( dir ) QListBox_SourceDirs->insertItem( dir );
	QListBox_SourceDirs->insertItem( dir );
}

//Right click on QListBoxItem
void MasterCD::slot_deleteTrack(){
	QListBox_SourceDirs->removeItem( QListBox_SourceDirs->currentItem() );
}

//'ImageFile-Button' clicked
void MasterCD::slot_setImageFile(){
	QLineEdit_ImageFile->setText( KFileDialog::getSaveFileName() );
}

//'BootImage-Button' clicked
void MasterCD::slot_setBootImage(){
	QLineEdit_BootImage->setText( KFileDialog::getOpenFileName() );
}

//'Image-Type Button' clicked
void MasterCD::slot_imageType(){
	ImageType *sdlg = new ImageType;
	sdlg->show();

	//Read Image-Type from config file and display it
	KConfig *config = kapp->config();
	config->setGroup("Image-Type");
	QLabel_ImageType->setText( config->readEntry( "Name" ) );
}

//'QCheckBox_BootableCD' clicked , toggle BootableCD-QLineEdit and -Button
void MasterCD::slot_toggleBootableCD(){
	QPushButton_BootImage->setEnabled( QCheckBox_BootableCD->isChecked() ? true:false );
	QLineEdit_BootImage->setEnabled( QCheckBox_BootableCD->isChecked() ? true:false );
	QLineEdit_BootImage->setText( "" );
}

//'QCheckBox_Burn on-the-fly' clicked
void MasterCD::slot_toggle_onthefly(){
	QCheckBox_Multisession->setEnabled( QCheckBox_onthefly->isChecked() ? false:true );
	QCheckBox_Multisession->setChecked( false );
	QPushButton_ImageFile->setEnabled( QCheckBox_onthefly->isChecked() ? false:true );
	QLineEdit_ImageFile->setEnabled( QCheckBox_onthefly->isChecked() ? false:true );
	QLineEdit_ImageFile->setText( "" );
	QCheckBox_LeaveImage->setEnabled( QCheckBox_onthefly->isChecked() ? false:true );
	QCheckBox_LeaveImage->setChecked( false );
	QCheckBox_CreateImage->setEnabled( QCheckBox_onthefly->isChecked() ? false:true );
	QCheckBox_CreateImage->setChecked( QCheckBox_onthefly->isChecked() ? false:true );
	QCheckBox_BootableCD->setEnabled( QCheckBox_onthefly->isChecked() ? false:true );
	QCheckBox_BootableCD->setChecked( false );
	if ( ! QCheckBox_nofix->isEnabled() ) QCheckBox_nofix->setEnabled( true );
	if ( QCheckBox_BootableCD->isChecked() ){
		QPushButton_BootImage->setEnabled( false );
		QLineEdit_BootImage->setEnabled( false );
		QLineEdit_BootImage->setText( "" );
	}
}

//'QCheckBox_WriteCD' clicked
void MasterCD::slot_toggleWriteCD(){
	QCheckBox_EjectCD->setEnabled( QCheckBox_WriteCD->isChecked() ? true:false );
	QCheckBox_EjectCD->setChecked( false );
	QCheckBox_nofix->setEnabled( QCheckBox_WriteCD->isChecked() ? true:false );
	QCheckBox_nofix->setChecked( false );
	QCheckBox_Simulate->setEnabled( QCheckBox_WriteCD->isChecked() ? true:false );
	QCheckBox_Simulate->setChecked( false );
	QCheckBox_Force->setEnabled( QCheckBox_WriteCD->isChecked() ? true:false );
	QCheckBox_Force->setChecked( false );
	QCheckBox_ignoresize->setEnabled( QCheckBox_WriteCD->isChecked() ? true:false );
	QCheckBox_ignoresize->setChecked( false );
	QCheckBox_LeaveImage->setChecked( false );
	if ( ! QCheckBox_onthefly->isChecked() ) QCheckBox_LeaveImage->setEnabled( QCheckBox_WriteCD->isChecked() ? true:false );
}

//'QCheckBox_CreateImage' clicked
void MasterCD::slot_toggleCreateImage(){
	QCheckBox_BootableCD->setEnabled( QCheckBox_CreateImage->isChecked() ? true:false );
	QCheckBox_BootableCD->setChecked( false );
	QPushButton_BootImage->setEnabled( false );
	QLineEdit_BootImage->setEnabled( false );
	QLineEdit_BootImage->setText( "" );
	QListBox_SourceDirs->setEnabled( QCheckBox_CreateImage->isChecked() ? true:false );
	QListBox_SourceDirs->clear();
	QPushButton_SourceDir->setEnabled( QCheckBox_CreateImage->isChecked() ? true:false );
	QPushButton_DeleteTrack->setEnabled( QCheckBox_CreateImage->isChecked() ? true:false );
}

//'QCheckBox_Multisession' clicked
void MasterCD::slot_multisession(){
	QCheckBox_nofix->setEnabled( QCheckBox_Multisession->isChecked() ? false:true );
	if ( QCheckBox_nofix->isChecked() ) QCheckBox_nofix->setChecked( false );
}

//'Start-Button' clicked
void MasterCD::slot_start(){
	//Check for QCheckBox_onthefly
	if ( QCheckBox_onthefly->isChecked() ) {
		//Check for SourceDirs,
		if ( QListBox_SourceDirs->count() == 0 ) {
			KMessageBox::error( 0, i18n("Master CD:\nNot valid Source-Dirs !") );
			return;
		}
		//Check QCheckBox_WriteCD
		if ( ! QCheckBox_WriteCD->isChecked() ) {
			KMessageBox::error( 0, i18n("Master CD:\nYou must enable 'WriteCD' !") );
			return;
		}
		slot_PrintSize();
		return;
	}

	//--CheckBox_CreateImage is true--
	if ( QCheckBox_CreateImage->isChecked() ) {
		//If not valid Source-Dirs specified
		if ( QListBox_SourceDirs->count() == 0 ) {
			KMessageBox::error( 0, i18n("Master CD:\nNot valid Source-Dirs !") );
			return;
		}
		//If not valid Image-File specified
		if( ! strlen( QLineEdit_ImageFile->text() ) ) {
			KMessageBox::error( 0, i18n("Master CD:\nNot valid Image-File !") );
			return;
		}
		//If QCheckBox_BootableCD enabled and not valid Boot-Image specified
		if( QCheckBox_BootableCD->isChecked() && ! strlen( QLineEdit_BootImage->text() ) ) {
			KMessageBox::error( 0, i18n("Master CD:\nNot valid Boot-Image !") );
			return;
		}
		//Check for ImageType
		if( ! strlen( QLabel_ImageType->text() ) ) {
			KMessageBox::error( 0, i18n("Master CD:\nNot valid Image-Type !") );
			return;
		}
		//If multisession
		if ( QCheckBox_Multisession->isChecked() ) slot_MSInfo();

		//Create the CD-Image
		else slot_makeImage();
	}

	//--CheckBox_WriteCD is true--
	if ( QCheckBox_WriteCD->isChecked() && ! QCheckBox_CreateImage->isChecked() ) {
		//If the Imagefile exists, than write the CD
		if ( ! QFile::exists( QLineEdit_ImageFile->text() ) ) {
			KMessageBox::error( 0, i18n("Master CD:\nThe Imagefile does not exists !") );
			return;
		}
		//Write the CD
		slot_writeCD();
		}
}

//---Create CD-Image with mkisofs---
void MasterCD::slot_makeImage(){
	KConfig *config = kapp->config();

	//KProcess arguments
	KProcess *process = new KProcess();
	process->clearArguments();
 	(*process) << "mkisofs" << "-f";

	//Image-Type
	config->setGroup("Image-Type");
	if ( config->readBoolEntry( "Allowlongfilenames" ) ) (*process) << "-l";
	if ( config->readBoolEntry( "Rockridgeextensions" ) ) (*process) << "-R";
	if ( config->readBoolEntry( "Rockridgewithanonymity" ) ) (*process) << "-r";
	if ( config->readBoolEntry( "Jolietextensions" ) ) (*process) << "-J";
	if ( config->readBoolEntry( "Omittrailingperiods" ) ) (*process) << "-d";
	if ( config->readBoolEntry( "Dontusedeepdirrelocations" ) ) (*process) << "-D";
	if ( config->readBoolEntry( "Allowfilesbeginningwithadot" ) ) (*process) << "-L";
	if ( config->readBoolEntry( "Omitisoversionnumbers" ) ) (*process) << "-N";
	if ( config->readBoolEntry( "Dontincludebackupfiles" ) ) (*process) << "-no-bak";
	if ( config->readBoolEntry( "DontuseRR" ) ) (*process) << "-no-rr";
	if ( config->readBoolEntry( "CreateTRANS_TBLfiles" ) ) (*process) << "-T";
	if ( config->readBoolEntry( "HideRR_MOVEDdir" ) ) (*process) << "-hide-rr-moved";
	if ( config->readBoolEntry( "FollowSymbolicLinks" ) ) (*process) << "-f";
	if ( config->readBoolEntry( "HideTRANS_TBLfiles" ) ) (*process) << "-hide-joliet-trans-tbl";
	if ( config->readBoolEntry( "Untranslatedfiles" ) ) (*process) << "-U";

	//Arguments for multisession
	if ( QCheckBox_Multisession->isChecked() && existsSession == true ) {
		char tmp[50], dev[10], outdev[10];
		char *tmpdev;
		sprintf( tmp, "%d,%d", MS_Start, MS_End );
	 	(*process) << "-C" << tmp;
		config->setGroup("CD-Writer");
		strcpy(dev, config->readEntry("Device"));
		printf("%s\n", strtok(dev, "="));
		while ( (tmpdev=strtok(NULL, "=")) != NULL)
		strcpy(outdev, tmpdev);
		(*process) << "-M" << outdev;
	}

	//Bootable CD
	if( QCheckBox_BootableCD->isChecked() ) (*process) << "-b" << QLineEdit_BootImage->text() << "-c" << "boot_catalog";

	//CD-Identification
	if( QLineEdit_AppID->text() ) (*process) << "-A" << QLineEdit_AppID->text();
	if( QLineEdit_Publisher->text() ) (*process) << "-P" << QLineEdit_Publisher->text();
	if( QLineEdit_Preparer->text() ) (*process) << "-p" << QLineEdit_Preparer->text();
	if( QLineEdit_VolumeID->text() ) (*process) << "-V" << QLineEdit_VolumeID->text();

	//Image-File
	(*process) << "-o" << QLineEdit_ImageFile->text();

	//Add the source-dirs to the process
	int trackcount = 0;
	int tc = QListBox_SourceDirs->count();
	do {
		(*process) << QListBox_SourceDirs->text( trackcount );
		trackcount++;
	}
	while ( trackcount < tc );

	//Connect the Abort-Button from ImgProOut-Dialog to Abort-Handler
	connect( imgprocoutdlg.Button, SIGNAL(clicked()), this, SLOT(abort_handler()));

	//Connect the 'processExited' Signal to the 'slotBurnProcessFinished' Slot
	connect( process, SIGNAL(processExited(KProcess *)), this, SLOT(slot_makeImageProcessExited(KProcess *)));

	//Connect with slotRecStderr to update KProgress (Write- / Buffer-Status) and StatusLine
	connect( process, SIGNAL(receivedStdout (KProcess *, char *, int) ), this, SLOT(slot_makeImageStdout(KProcess *, char *, int) ));
	connect( process, SIGNAL(receivedStderr (KProcess *, char *, int) ), this, SLOT(slot_makeImageStdout(KProcess *, char *, int) ));

	//Start KProcess
	if( ! process->start( KProcess::NotifyOnExit, KProcess::AllOutput ) ) KMessageBox::error( 0, i18n("Create Image:\nCould not start mkisofs !") );

	imgprocoutdlg.StatusLabel -> setText( i18n("Creating CD-Image...") );
	imgprocoutdlg.Button->setEnabled( false );
	if ( ! imgprocoutdlg.isVisible() ) imgprocoutdlg.show();
}

//---Write CD with cdrecord---
void MasterCD::slot_writeCD(){
	KConfig *config = kapp->config();

	//KProcess argument
	KProcess *process = new KProcess();
	process->clearArguments();
	(*process) << "cdrecord" << "-v";

	//Set the CD-Writer first
	config->setGroup("CD-Writer");
	(*process) << config->readEntry("Device");

	//Set Burn-Proof
	if( config->readBoolEntry("BurnProof") ) (*process) << "driveropts=burnproof";

	//Set CD-Writer Options to KProcess
	if( QCheckBox_EjectCD->isChecked() ) (*process) << "-eject";
	if( QCheckBox_Force->isChecked() ) (*process) << "-force";
	if( QCheckBox_Multisession->isChecked() ) (*process) << "-multi";
	if( QCheckBox_Simulate->isChecked() ) (*process) << "-dummy";
	if( QCheckBox_nofix->isChecked() ) (*process) << "-nofix";
	if( QCheckBox_ignoresize->isChecked() ) (*process) << "-ignsize";

	//Set CD-RW Options to KProcess
	switch( QComboBox_CDRWBlanking->currentItem() )
	{
	 	case 1:	(*process) << "blank=all";
						break;
	 	case 2: (*process) << "blank=fast";
						break;
	 	case 3: (*process) << "blank=track";
						break;
	 	case 4: (*process) << "blank=unreserve";
						break;
	 	case 5: (*process) << "blank=trtrail";
						break;
	 	case 6: (*process) << "blank=unclose";
						break;
		case 7: (*process) << "blank=session";
						break;
	}

	//Set CD-Writer Speed to KProcess
	switch( QComboBox_WriterSpeed -> currentItem() )
	{
   	case 0:	(*process) << "speed=1";
						break;
   	case 1:	(*process) << "speed=2";
						break;
   	case 2:	(*process) << "speed=4";
						break;
   	case 3:	(*process) << "speed=6";
						break;
   	case 4:	(*process) << "speed=8";
						break;
   	case 5:	(*process) << "speed=10";
						break;
   	case 6:	(*process) << "speed=12";
						break;
   	case 7:	(*process) << "speed=16";
						break;
   	case 8:	(*process) << "speed=20";
						break;
	}

	//Set the Image
	(*process) << QLineEdit_ImageFile->text();

	//Connect the Abort-Button from ImgProOut-Dialog to Abort-Handler
	connect( procoutdlg.QPushButton_OK, SIGNAL(clicked()), this, SLOT(abort_handler()));

	//Connect the 'processExited' Signal to the 'slotProcessFinished' Slot
	connect( process, SIGNAL(processExited(KProcess *)), this, SLOT(slot_burnProcessExited(KProcess *)));

	//Connect with slotRecStderr to update KProgress (Write- / Buffer-Status) and StatusLine
	connect( process, SIGNAL(receivedStdout (KProcess *, char *, int) ), this, SLOT(slot_burnRecStderr (KProcess *, char *, int) ));
	connect( process, SIGNAL(receivedStderr (KProcess *, char *, int) ), this, SLOT(slot_burnRecStderr (KProcess *, char *, int) ));

	//Start KProcess
	if( ! process->start( KProcess::NotifyOnExit, KProcess::AllOutput ) )
		{
		KMessageBox::error( 0, i18n("Write CD:\nCould not start cdrecord !") );
		return;
		}

	procoutdlg.QPushButton_OK->setEnabled( false );
	if ( ! procoutdlg.isVisible() ) procoutdlg.show();
}

//'Calculate-Button' clicked
void MasterCD::slot_calculateSize() {
	//If not valid Source-Dirs specified
	if( QListBox_SourceDirs->count() == 0 ) {
		KMessageBox::error( 0, i18n("Calculate size:\nNot valid Source-Dirs !") );
		return;
	}

	QPushButton_Start->setEnabled( false );
	QPushButton_Quit->setEnabled( false );
	QPushButton_Calculate->setText( "working..." );
	setCaption( i18n( "KOnCD - Master CD: Calculating CD-Image size" ) );

	//Start KProcess for calculating ISO-Image size with mkisofs
	KProcess *process = new KProcess();
	process->clearArguments();
 	(*process) << "mkisofs" << "-print-size" << "-f";

	//Add the source-dirs to the process
	int trackcount = 0;
	int tc = QListBox_SourceDirs->count();
	do {
		(*process) << QListBox_SourceDirs->text( trackcount );
		trackcount++;
	}
	while ( trackcount < tc );

	//Connect the 'processExited' Signal to the 'slotProcessFinished' Slot
	connect( process, SIGNAL(processExited(KProcess *)), this, SLOT(slot_calculateProcessExited(KProcess *)));

	//Connect with all Signals from KProcess to Slot 'slotRecStderr'
	connect( process, SIGNAL(receivedStdout (KProcess *, char *, int) ), this, SLOT(slot_calculateRecStderr (KProcess *, char *, int) ));
	connect( process, SIGNAL(receivedStderr (KProcess *, char *, int) ), this, SLOT(slot_calculateRecStderr (KProcess *, char *, int) ));

	//Start KProcess
	if( ! process->start( KProcess::NotifyOnExit, KProcess::AllOutput ) ) KMessageBox::error( 0, i18n("Calculate size:\nCould not start mkisofs !") );

}

//KProcess finished
void MasterCD::slot_calculateProcessExited(KProcess *rcproc) {
	//Check, if KProcess exited normally...
	if( rcproc->normalExit() ) {
 			if( rcproc->exitStatus() != 0 ) {
				QString statusout;
				statusout.sprintf( "Exit status: %d", rcproc->exitStatus() );
				KMessageBox::error( 0, i18n("Calculate:\nAn error is occurred !") );
				QPushButton_Start->setEnabled( true );
				QPushButton_Quit->setEnabled( true );
				QPushButton_Calculate->setText(i18n("&Calculate"));
				setCaption( i18n( "KOnCD - Master CD" ) );
				return;
			}
		QPushButton_Start->setEnabled( true );
		QPushButton_Quit->setEnabled( true );
		QPushButton_Calculate->setText(i18n("&Calculate"));
		setCaption( i18n( "KOnCD - Master CD" ) );
	}
}

//---Write CD on-the-fly---
void MasterCD::slot_writeCD_onthefly(){
	char tmp[50];

	//KShellProcess
	KShellProcess *shellProcess = new KShellProcess();

	//Set args for mkisofs
 	(*shellProcess) << "mkisofs" << "-f";

	//Image-Type
	KConfig *config = kapp->config();
	config->setGroup("Image-Type");
	if ( config->readBoolEntry( "Allowlongfilenames" ) ) (*shellProcess) << "-l";
	if ( config->readBoolEntry( "Rockridgeextensions" ) ) (*shellProcess) << "-R";
	if ( config->readBoolEntry( "Rockridgewithanonymity" ) ) (*shellProcess) << "-r";
	if ( config->readBoolEntry( "Jolietextensions" ) ) (*shellProcess) << "-J";
	if ( config->readBoolEntry( "Omittrailingperiods" ) ) (*shellProcess) << "-d";
	if ( config->readBoolEntry( "Dontusedeepdirrelocations" ) ) (*shellProcess) << "-D";
	if ( config->readBoolEntry( "Allowfilesbeginningwithadot" ) ) (*shellProcess) << "-L";
	if ( config->readBoolEntry( "Omitisoversionnumbers" ) ) (*shellProcess) << "-N";
	if ( config->readBoolEntry( "Dontincludebackupfiles" ) ) (*shellProcess) << "-no-bak";
	if ( config->readBoolEntry( "DontuseRR" ) ) (*shellProcess) << "-no-rr";
	if ( config->readBoolEntry( "CreateTRANS_TBLfiles" ) ) (*shellProcess) << "-T";
	if ( config->readBoolEntry( "HideRR_MOVEDdir" ) ) (*shellProcess) << "-hide-rr-moved";
	if ( config->readBoolEntry( "FollowSymbolicLinks" ) ) (*shellProcess) << "-f";
	if ( config->readBoolEntry( "HideTRANS_TBLfiles" ) ) (*shellProcess) << "-hide-joliet-trans-tbl";
	if ( config->readBoolEntry( "Untranslatedfiles" ) ) (*shellProcess) << "-U";

	//Add CD-Identification
	if( QLineEdit_VolumeID->text() ){
		QString volumeid;
		volumeid.append( "-V '" );
		volumeid.append( QLineEdit_VolumeID->text() );
		volumeid.append( "'" );
		(*shellProcess) << volumeid;
	}

	if( QLineEdit_Publisher->text() ){
		QString publisher;
		publisher.append( "-P '" );
		publisher.append( QLineEdit_Publisher->text() );
		publisher.append( "'" );
		(*shellProcess) << publisher;
	}

	if( QLineEdit_Preparer->text() ){
		QString preparer;
		preparer.append( "-p '" );
		preparer.append( QLineEdit_Preparer->text() );
		preparer.append( "'" );
		(*shellProcess) << preparer;
	}

	if( QLineEdit_AppID->text() ){
		QString appid;
		appid.append( "-A '" );
		appid.append( QLineEdit_AppID->text() );
		appid.append( "'" );
		(*shellProcess) << appid;
	}

	//Add the source-dirs to the process
	int trackcount = 0;
	int tc = QListBox_SourceDirs->count();
	do {
		(*shellProcess) << QListBox_SourceDirs->text( trackcount );
		trackcount++;
	}
	while ( trackcount < tc );

	//For KShellProcess
	(*shellProcess) << "|";

	//Set args for cdrecord
	(*shellProcess) << "cdrecord" << "-v";

	//Set the CD-Writer first
	config->setGroup("CD-Writer");
	(*shellProcess) << config->readEntry("Device");

	//Set Burn-Proof
	if( config->readBoolEntry("BurnProof") ) (*shellProcess) << "driveropts=burnproof";

	//Set CD-Writer Options to KProcess
	if( QCheckBox_EjectCD->isChecked() ) (*shellProcess) << "-eject";
	if( QCheckBox_Force->isChecked() ) (*shellProcess) << "-force";
	if( QCheckBox_Simulate->isChecked() ) (*shellProcess) << "-dummy";
	if( QCheckBox_nofix->isChecked() ) (*shellProcess) << "-nofix";
	if( QCheckBox_ignoresize->isChecked() ) (*shellProcess) << "-ignsize";

	//Set CD-RW Options to KProcess
	switch( QComboBox_CDRWBlanking->currentItem() )
	{
	 	case 1:	(*shellProcess) << "blank=all";
						break;
	 	case 2: (*shellProcess) << "blank=fast";
						break;
	 	case 3: (*shellProcess) << "blank=track";
						break;
	 	case 4: (*shellProcess) << "blank=unreserve";
						break;
	 	case 5: (*shellProcess) << "blank=trtrail";
						break;
	 	case 6: (*shellProcess) << "blank=unclose";
						break;
		case 7: (*shellProcess) << "blank=session";
						break;
	}

	//Set CD-Writer Speed to KProcess
	switch( QComboBox_WriterSpeed -> currentItem() )
	{
   	case 0:	(*shellProcess) << "speed=1";
						break;
   	case 1:	(*shellProcess) << "speed=2";
						break;
   	case 2:	(*shellProcess) << "speed=4";
						break;
   	case 3:	(*shellProcess) << "speed=6";
						break;
   	case 4:	(*shellProcess) << "speed=8";
						break;
   	case 5:	(*shellProcess) << "speed=10";
						break;
   	case 6:	(*shellProcess) << "speed=12";
						break;
   	case 7:	(*shellProcess) << "speed=16";
						break;
   	case 8:	(*shellProcess) << "speed=20";
						break;
	}

	//Set filesystem size
	sprintf( tmp, "tsize=%d", FS_Size );
	(*shellProcess) << tmp;

	//Last arg
	(*shellProcess) << "-";

	//Connect the Abort-Button from ImgProOut-Dialog to Abort-Handler
	connect( procoutdlg.QPushButton_OK, SIGNAL(clicked()), this, SLOT(abort_handler()));

	//Connect the 'processExited' Signal to the 'slotProcessFinished' Slot
	connect( shellProcess, SIGNAL(processExited(KProcess *)), this, SLOT(slot_burnProcessExited(KProcess *)));

	//Connect with slotRecStderr to update KProgress (Write- / Buffer-Status) and StatusLine
	connect( shellProcess, SIGNAL(receivedStdout (KProcess *, char *, int) ), this, SLOT(slot_burnRecStderr (KProcess *, char *, int) ));
	connect( shellProcess, SIGNAL(receivedStderr (KProcess *, char *, int) ), this, SLOT(slot_burnRecStderr (KProcess *, char *, int) ));

	//Start KProcess
	if( ! shellProcess->start( KProcess::NotifyOnExit, KProcess::AllOutput ) )
		{
		KMessageBox::error( 0, i18n("Write CD:\nCould not start cdrecord !") );
		return;
		}

	procoutdlg.StatusLabel -> setText( i18n("Prepare for write...") );
	procoutdlg.QPushButton_OK->setEnabled( false );
}

//KProcess outout analysis
void MasterCD::slot_calculateRecStderr( KProcess *, char *buffer, int buflen) {
	char *c, buf[1024];

	if( !buflen ) {
		return;
	}
	bzero( buf, sizeof( buf ) );
	strncpy( buf, buffer, buflen > (int) sizeof( buf ) ? sizeof(buf) - 1 : buflen );
	if( strstr( buf, "written =" ) ) {
		if( (c = rindex( buf, ' ' ) ) ) {		
			int s = atoi( c ) * 2 / 1024;
			char tmp[50];
			sprintf( tmp, "%d MB", s );
			QLabel_DataSize->setText( tmp );

			if( s > 650 && s < 730 ) {
				KMessageBox::sorry( 0, i18n("Calculate size:\nThe CD-Image becomes over 650 MB !\n"
																  "The 'Ignore medium size' options must be enabled\n"
																  "and use a CD-R 700 or higher !" ) );
				QCheckBox_ignoresize->setChecked( true );
				return;
			}
			if( s > 730 ) {
				KMessageBox::sorry( 0, i18n("Calculate size:\nThe CD-Image becomes over 730 MB !\n"
   																   "You can't write that on a CD-R !" ) );
				return;
			}

		}
	}
}

//KProcess outout analysis
void MasterCD::slot_burnRecStderr(KProcess *, char *buffer, int buflen) {
	char *c, buf[1024];
	int wrote, total, load;

	if( !buflen ) {
		return;
	}
	bzero( buf, sizeof( buf ) );
	strncpy( buf, buffer, buflen > (int) sizeof( buf ) ? sizeof(buf) - 1 : buflen );
	//Add cdrecord Messages to the Output-Window
	procoutdlg.Output->insertLine( buf );
	//Go to the End of the Output-Text
	procoutdlg.Output->setCursorPosition(procoutdlg.Output->numLines(),0,false);

	printf( buf );

	//Blanking a CD-RW
	if( strstr( buf, "Blanking" )) {
		procoutdlg.StatusLabel -> setText( i18n("Blanking the CDRW...") );
	}

	//Burn without Fixating
	if( strstr( buf, "Writing  time:" )) {
		procoutdlg.StatusLabel -> setText( i18n("Burn-Process complete.") );
	}

	//Fixating the CD-R
	if( strstr( buf, "Fixating..." )) {
		procoutdlg.StatusLabel -> setText( i18n("Writing TOC...") );
	}

	//	//Burn with Fixating
	if( strstr( buf, "Fixating time:" )) {
		procoutdlg.StatusLabel -> setText( i18n("Burn-Process complete.") );
   	}
	
	if( strstr( buf, "MB written" )) {
		c = index( buf, ':' );
		if( c && *c ) {
			c = strtok( c, ":\r\t " );
			if( c ) {
				wrote = atoi( c );
				c = strtok( NULL, "\r\t " );
				c = strtok( NULL, "\r\t " );
				if( c ) {
					total = atoi( c );
					c = strtok( NULL, "\r" );
					c = strstr( c, "fifo " );
					if( c ) {
						c = strtok( c + 5, "%\r\t " );
						if( c ) {
							load = atoi( c );
							procoutdlg.WriteProgress->setProgress( total ? wrote * 100 / total : 0 );
							procoutdlg.BufferProgress->setProgress( load );

							if( ( total == wrote ) ) {
								procoutdlg.StatusLabel->setText( i18n("Writing TOC...") );
							}
							
							if( wrote ) {
								if( QCheckBox_Simulate->isChecked() ) {
									procoutdlg.StatusLabel -> setText( i18n("Writing to CD in dummy mode...") );
								}
								else {
									procoutdlg.StatusLabel -> setText( i18n("Writing to CD...") );
								}
							}

						}
					}
				}
			}
		}		
	}	

}

//KProcess finished
void MasterCD::slot_burnProcessExited(KProcess *rcproc) {
	//Check, if KProcess exited normally...
	if( rcproc->normalExit() ) {
 			if( rcproc->exitStatus() != 0 ) {
				switch ( rcproc->exitStatus() )
				{
					case 13:		procoutdlg.StatusLabel -> setText( i18n( "Error: You don't have enough permissions !" ) );
										break;

					case 19:		procoutdlg.StatusLabel -> setText( i18n( "Error: Can't open SCSI-Driver !" ) );
										break;

					case 254:	procoutdlg.StatusLabel -> setText( i18n( "Error: Corrupt CD in the Writer-Device !" ) );
										break;

					case 255:	procoutdlg.StatusLabel -> setText( i18n( "Error: No CD in the Writer-Device !" ) );
										break;
				}
				QString statusout;
				statusout.sprintf( "Exit status: %d", rcproc->exitStatus() );
				KMessageBox::error( 0, i18n("Write CD:\nAn error is occurred !\n" + statusout) );
				procoutdlg.QPushButton_OK->setText( "&OK" );
				procoutdlg.QPushButton_OK->setEnabled( true );
				return;
			}
		//If 'Leave Image' CheckBox false, delete the Imagefile
		if ( rcproc->exitStatus() == 0 && ! QCheckBox_LeaveImage->isChecked() && ! QCheckBox_onthefly->isChecked() ) QFile::remove( QLineEdit_ImageFile->text() );
		procoutdlg.QPushButton_OK->setText( "&OK" );
		procoutdlg.QPushButton_OK->setEnabled( true );
	}
}

//KProcess outout analysis
void MasterCD::slot_makeImageStdout(KProcess *, char *buffer, int buflen) {
	char *c, buf[1024];

	if( !buflen ) {
		return;
	}
	bzero( buf, sizeof( buf ) );
	strncpy( buf, buffer, buflen > (int) sizeof( buf ) ? sizeof(buf) - 1 : buflen );

	//Add cdrecord Messages to the Output-Window
	imgprocoutdlg.Output->insertLine( buf );
	//Go to the End of the Output-Text
	imgprocoutdlg.Output->setCursorPosition(imgprocoutdlg.Output->numLines(),0,false);

	if( strstr( buf, "extents written" ) ) {
		imgprocoutdlg.ImgProgress->setProgress( 100 );
		return;
	}

	if( strstr( buf, "% done" ) ) {
		if( (c = strtok( buf, "%" ) ) ) {
			imgprocoutdlg.ImgProgress->setProgress( atoi( c ) );
		}
	}
}

//KProcess finished
void MasterCD::slot_makeImageProcessExited(KProcess *rcproc) {
	//Check, if KProcess exited normally...
	if( rcproc->normalExit() ) {
 			if( rcproc->exitStatus() != 0 ) {
				if( rcproc->exitStatus() == 13 ) imgprocoutdlg.StatusLabel -> setText( i18n( "Error: You don't have enough permissions !" ) );
				QString statusout;
				statusout.sprintf( "Exit status: %d", rcproc->exitStatus() );
				KMessageBox::error( 0, i18n("Create cd image:\nAn error is occurred !\n" + statusout) );
				imgprocoutdlg.Button->setText( "&OK" );
				imgprocoutdlg.Button->setEnabled( true );
				return;
			}
			//If CheckBox 'Write CD' checked, close the dialog and write the CD
			else if ( QCheckBox_WriteCD->isChecked() ){
				imgprocoutdlg.StatusLabel -> setText( i18n("Image creating completed.") );
				//Close ProcessDialog
				imgprocoutdlg.close();
				slot_writeCD();
			}
			else {
				imgprocoutdlg.StatusLabel -> setText( i18n("Image creating completed.") );
				imgprocoutdlg.Button->setText( "&OK" );
				imgprocoutdlg.Button->setEnabled( true );
			}
	}
}

//KProcess for multisession info
void MasterCD::slot_MSInfo() {
	KConfig *config = kapp->config();

	//KProcess arguments
	KProcess *process = new KProcess();
	process->clearArguments();
 	(*process) << "cdrecord";
	config->setGroup("CD-Writer");
	(*process) << config->readEntry("Device") << "-msinfo";

	//Connect the Abort-Button from ImgProOut-Dialog to Abort-Handler
	connect( imgprocoutdlg.Button, SIGNAL(clicked()), this, SLOT(abort_handler()));

	//Connect the 'processExited' Signal
	connect( process, SIGNAL(processExited(KProcess *)), this, SLOT( slot_MSInfoExited( KProcess * ) ));

	//Connect with slotRecStderr to update KProgress
	connect( process, SIGNAL(receivedStdout(KProcess *, char *, int) ), this, SLOT( slot_MSOutput(KProcess *, char *, int ) ));
	connect( process, SIGNAL(receivedStderr(KProcess *, char *, int) ), this, SLOT( slot_MSOutput(KProcess *, char *, int ) ));

	//Start KProcess
	if( ! process->start( KProcess::NotifyOnExit, KProcess::AllOutput ) ) KMessageBox::error( 0, i18n("MSInfo:\nCould not start cdrecord !") );

	imgprocoutdlg.StatusLabel -> setText( i18n( "Retrieve multisession info..." ) );
	if ( ! imgprocoutdlg.isVisible() ) imgprocoutdlg.show();
}

//KProcess Analyse for msinfo output
void MasterCD::slot_MSOutput( KProcess *, char *s, int n ){
	char *c, *b, buf[1024];

	if( !n ) return;
	
	bzero( buf, sizeof( buf ) );
	strncpy( buf, s, n > (int) sizeof( buf ) ? sizeof(buf) - 1 : n );

	//Check for exists session
	if( strstr( buf, "status: 0x2 (CHECK CONDITION)" )) {
		imgprocoutdlg.StatusLabel->setText( i18n( "None session available, creating image..." ) );
		existsSession = false;
  }
	else {
		imgprocoutdlg.StatusLabel->setText( i18n( "Session available, creating image..." ) );
		existsSession = true;
	}

	c = strtok( buf, "\n" );
	while( c ) {
  	if( (b = strchr( c, ',' )) && isdigit( *c ) ) {
  		b = strtok( c, ",\n" );
  		if( b ) MS_Start = atoi( b );
  		b = strtok( NULL, ",\n" );
  		if( b ) MS_End = atoi( b );
  	}
		c = strtok( NULL, "\n" );	
	}
}

//KProcess 'msinfo' finished
void MasterCD::slot_MSInfoExited(KProcess *rcproc) {
	//Check, if KProcess exited normally...
	if( rcproc->normalExit() ) {
 			if( rcproc->exitStatus() != 0 ) {
				switch ( rcproc->exitStatus() )
				{
					case 13:		imgprocoutdlg.StatusLabel -> setText( i18n( "Error: You don't have enough permissions !" ) );
										break;

					case 19:		imgprocoutdlg.StatusLabel -> setText( i18n( "Error: Can't open SCSI-Driver !" ) );
										break;

					case 254:	imgprocoutdlg.StatusLabel -> setText( i18n( "Error: Corrupt CD in the Writer-Device !" ) );
										break;

					case 255:	imgprocoutdlg.StatusLabel -> setText( i18n( "Error: No CD in the Writer-Device !" ) );
										break;
				}
				QString statusout;
				statusout.sprintf( "Exit status: %d", rcproc->exitStatus() );
				KMessageBox::error( 0, i18n("MSInfo:\nAn error is occurred !\n" + statusout) );
				imgprocoutdlg.Button->setText( "&OK" );
				return;
			}
		//Create the Image
		slot_makeImage();
	}
}

//KProcess for printsize
void MasterCD::slot_PrintSize() {
	//KProcess arguments
	KProcess *process = new KProcess();
	process->clearArguments();
 	(*process) << "mkisofs" << "-quiet" << "-print-size";

	//Add the source-dirs to the process
	int trackcount = 0;
	int tc = QListBox_SourceDirs->count();
	do {
		(*process) << QListBox_SourceDirs->text( trackcount );
		trackcount++;
	}
	while ( trackcount < tc );

	//Connect the Abort-Button from ImgProOut-Dialog to Abort-Handler
	connect( imgprocoutdlg.Button, SIGNAL(clicked()), this, SLOT(abort_handler()));

	//Connect the 'processExited' Signal
	connect( process, SIGNAL(processExited(KProcess *)), this, SLOT( slot_PrintSizeExited( KProcess * ) ));

	//Connect with slotRecStderr to update KProgress
	connect( process, SIGNAL(receivedStdout(KProcess *, char *, int) ), this, SLOT( slot_PrintSizeOutput(KProcess *, char *, int ) ));
	connect( process, SIGNAL(receivedStderr(KProcess *, char *, int) ), this, SLOT( slot_PrintSizeOutput(KProcess *, char *, int ) ));

	//Start KProcess
	if( ! process->start( KProcess::NotifyOnExit, KProcess::AllOutput ) ) KMessageBox::error( 0, i18n("PrintSize:\nCould not start cdrecord !") );

	procoutdlg.StatusLabel -> setText( i18n("Retrieve filesystem size...") );
	if ( ! procoutdlg.isVisible() ) procoutdlg.show();
}

//KProcess Analyse for print-size
void MasterCD::slot_PrintSizeOutput( KProcess *, char *s, int n ){
	char *c, buf[1024];

	if( !n ) return;
	
	bzero( buf, sizeof( buf ) );
	strncpy( buf, s, n > (int) sizeof( buf ) ? sizeof(buf) - 1 : n );

	if( strstr( buf, "written =" ) ) {
		if( (c = rindex( buf, ' ' ) ) ) {		
			FS_Size = atoi( c ) * 2048;
		}
	}
}

//KProcess 'print-size' finished
void MasterCD::slot_PrintSizeExited(KProcess *rcproc) {
	//Check, if KProcess exited normally...
	if( rcproc->normalExit() ) {
 			if( rcproc->exitStatus() != 0 ) {
				switch ( rcproc->exitStatus() )
				{
					case 13:		procoutdlg.StatusLabel -> setText( i18n( "Error: You don't have enough permissions !" ) );
										break;

					case 19:		procoutdlg.StatusLabel -> setText( i18n( "Error: Can't open SCSI-Driver !" ) );
										break;

					case 254:	procoutdlg.StatusLabel -> setText( i18n( "Error: Corrupt CD in the Writer-Device !" ) );
										break;

					case 255:	procoutdlg.StatusLabel -> setText( i18n( "Error: No CD in the Writer-Device !" ) );
										break;
				}
				QString statusout;
				statusout.sprintf( "Exit status: %d", rcproc->exitStatus() );
				KMessageBox::error( 0, i18n("Printsize:\nAn error is occurred !\n" + statusout) );
				procoutdlg.QPushButton_OK->setText( "&OK" );
				return;
			}
		//Write CD on-the-fly
		slot_writeCD_onthefly();
	}
}

//Abort-Handler
void MasterCD::abort_handler(){
	//Close dialogs
	if ( imgprocoutdlg.isVisible() ) imgprocoutdlg.close();
	if ( procoutdlg.isVisible() ) procoutdlg.close();
}

//'Quit-Button' clicked
void MasterCD::slot_quit(){

	//Write config
	KConfig *config = kapp->config();
	config->setGroup( "Config - mastercd" );

	config->writeEntry( "QCheckBox_BootableCD", QCheckBox_BootableCD->isChecked() );
	config->writeEntry( "QCheckBox_Multisession", QCheckBox_Multisession->isChecked() );
	config->writeEntry( "QCheckBox_LeaveImage", QCheckBox_LeaveImage->isChecked() );
	config->writeEntry( "QCheckBox_ignoresize", QCheckBox_ignoresize->isChecked() );
	config->writeEntry( "QCheckBox_Simulate", QCheckBox_Simulate->isChecked() );
	config->writeEntry( "QCheckBox_nofix", QCheckBox_nofix->isChecked() );
	config->writeEntry( "QCheckBox_EjectCD", QCheckBox_EjectCD->isChecked() );
	config->writeEntry( "QCheckBox_Force", QCheckBox_Force->isChecked() );

	config->writeEntry( "QComboBox_CDRWBlanking", QComboBox_CDRWBlanking->currentItem() );
	config->writeEntry( "QComboBox_WriterSpeed", QComboBox_WriterSpeed->currentItem() );

	config->writeEntry( "QLineEdit_VolumeID", QLineEdit_VolumeID->text() );
	config->writeEntry( "QLineEdit_Publisher", QLineEdit_Publisher->text() );
	config->writeEntry( "QLineEdit_Preparer", QLineEdit_Preparer->text() );
	config->writeEntry( "QLineEdit_AppID", QLineEdit_AppID->text() );

	config->writeEntry( "QLineEdit_ImageFile", QLineEdit_ImageFile->text() );
	config->writeEntry( "QLineEdit_BootImage", QLineEdit_BootImage->text() );

	close(true);
}
